<?php
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace IntiTest\Wp\Provider\Plugin;

use Inti\Provider\Service\ProviderManager;
use IntiTest\Wp\Provider\AbstractTestCase;
use IntiTest\Wp\Provider\Mock;
use WP_Post_Type;

use function get_post;
use function get_post_meta;
use function get_posts;
use function wp_count_posts;

/**
 * Class ProvidePostsTest
 *
 * @package IntiTest\Wp\Provider\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-bartog
 *
 * @runTestsInSeparateProcesses
 */
final class ProvidePostsTest extends AbstractTestCase
{
    private array $config = [
        ProviderManager::class => [
            'registered' => [
                Mock\PostProvider::class,
            ],
        ],
    ];

    public function testWillRegisterCustomPostType()
    {
        $type = get_post_type_object('custom_post');
        $this->assertInstanceOf(WP_Post_Type::class, $type);

        $this->assertEquals('Custom Posts', $type->label);
        $this->assertEquals('some description', $type->description);
        $this->assertTrue($type->public);
    }

    public function testCanProvidePosts()
    {
        $posts = get_posts([
            'post_type' => 'custom_post',
            'posts_per_page' => 20,
        ]);

        $this->assertCount(8, $posts);
    }

    public function testCanGetPostByIds()
    {
        $posts = get_posts([
            'p' => 100000004,
            'post_type' => 'custom_post',
        ]);

        $this->assertCount(1, $posts);
        $this->assertEquals('Fourth post', $posts[0]->post_title);
    }

    public function testCanGetMultiplePostsById()
    {
        $posts = get_posts([
            'post__in' => [
                100000002,
                '100000003',
            ],
            'post_type' => 'custom_post',
        ]);

        $this->assertCount(2, $posts);
        $this->assertEquals('Second post', $posts[0]->post_title);
        $this->assertEquals('Third post', $posts[1]->post_title);
    }

    public function testCanGetPostById()
    {
        $post = get_post(100000001);
        $this->assertEquals(100000001, $post->ID);
        $this->assertEquals('custom_post', $post->post_type);
        $this->assertEquals('First post', $post->post_title);
    }

    public function testCanSearchPosts()
    {
        $posts = get_posts([
            's' => 'Second',
            'post_type' => 'custom_post',
        ]);

        $this->assertCount(1, $posts);
        $this->assertEquals(100000002, $posts[0]->ID);
    }

    public function testCanGetPostBySlug()
    {
        $posts = get_posts([
            'name' => 'third-post',
            'post_type' => 'custom_post',
        ]);

        $this->assertCount(1, $posts);
        $this->assertEquals(100000003, $posts[0]->ID);
    }

    public function testCanCountPosts()
    {
        $count = wp_count_posts('custom_post');
        $this->assertEquals(8, $count->publish);
    }

    public function testCanProvideMetadataForProvidedPosts()
    {
        $post = get_post(100000001);
        $metadata = get_post_meta($post->ID, '_post_meta1', true);

        $this->assertEquals('post meta value 1-1', $metadata);
    }

    public function testCanExtractMetadataFromResults()
    {
        $post = get_post(100000001);
        $this->assertEquals('it is', get_post_meta($post->ID, '_extracted', true));
    }
}
